import pandas as pd
import numpy as np
import csv
import sys
import re

# function which calculates the cv(rmse)
def cvrmse_function(observed, predicted, N):
    error = observed - predicted
    sqerror = error ** 2
    sumsqerror = np.sum(sqerror)
    meansqerror = sumsqerror / N
    rmse = np.sqrt(meansqerror)
    cvrmse = 100 * rmse / np.mean(observed);
    return cvrmse

# function which calculates the normalized mean bias error
def nmbe_function(observed, predicted, N):
    error = predicted - observed
    sbe = np.sum(error)
    mbe = sbe / N
    nmbe = 100 * abs(mbe) / np.mean(observed)
    return nmbe

# function which calculates the max normalized (to [0, 100] absolute bias error
def maxe_function(observed, predicted):
    abs_error = 100 * np.absolute((predicted - observed) / observed)
    return np.max(abs_error)

# This script takes:
# sys.argv[1] – the full paths of the location of the project itself
# sys.argv[2] – the full path of the output folder of the project where the individual job folders are located
# sys.argv[3] – the list of the job IDs of the simulations that have been executed
# sys.argv[4] – the expected output table name, as defined by tableName. In the script, .csv should be appended to the table name before written to the file
# sys.argv[5] – Other arguments as specified in the scripts object in the RVX file

args = sys.argv[5].split(';')

cases = sys.argv[3].split(';')

refdata = pd.read_csv(sys.argv[1] + args[1]) #read data from the reference file (path passed from the script)

rowcolno = refdata.shape

in_file = sys.argv[2] + args[0] + ".csv"
indata = pd.read_csv(in_file)

#take heading row from the reference file
heading = list(refdata.ix[:1])
newheading = heading[0:3]
for h in heading [3:]:
    newheading += ['NMBE:' + re.sub('\\s*\[.*\].*', '[%]', h)]
    newheading += ['MAXE:' + re.sub('\\s*\[.*\].*', '[%]', h)]

# output file name from the parameter passed to the script
outname = sys.argv[4] # + '.csv'  if testing with jEPlus Py runner


with open(outname, "w", newline='') as outfile:
    output=csv.writer(outfile)
    output.writerow(newheading)
    counter = 0
    # for column in range(1, rowcolno[1]):
    for icase in cases:
        rmsedata = [counter]
        rmsedata.append(icase)
        rmsedata.append('na')
        for column in heading[3:]:
            observed = np.array(refdata.loc[:, column])
            predicted = np.array(indata.loc[indata.iloc[:, 1]==' ' + icase, column])
            rmsedata.append(nmbe_function(observed, predicted, rowcolno[0]))
            rmsedata.append(maxe_function(observed, predicted))
        output.writerow(rmsedata)
        counter = counter + 1
